/* Automatically generated nanopb header */
/* Generated by nanopb-0.3.9 at Thu Mar 29 04:30:08 2018. */

#ifndef PB_TENSORFLOW_FEATURE_PB_H_INCLUDED
#define PB_TENSORFLOW_FEATURE_PB_H_INCLUDED
#include "pb.h"

/* @@protoc_insertion_point(includes) */
#if PB_PROTO_HEADER_VERSION != 30
#error Regenerate this file with the current version of nanopb generator.
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* Struct definitions */
typedef struct _tensorflow_BytesList {
    pb_callback_t value;
/* @@protoc_insertion_point(struct:tensorflow_BytesList) */
} tensorflow_BytesList;

typedef struct _tensorflow_FeatureList {
    pb_callback_t feature;
/* @@protoc_insertion_point(struct:tensorflow_FeatureList) */
} tensorflow_FeatureList;

typedef struct _tensorflow_FeatureLists {
    pb_callback_t feature_list;
/* @@protoc_insertion_point(struct:tensorflow_FeatureLists) */
} tensorflow_FeatureLists;

typedef struct _tensorflow_Features {
    pb_callback_t feature;
/* @@protoc_insertion_point(struct:tensorflow_Features) */
} tensorflow_Features;

typedef struct _tensorflow_FloatList {
    pb_callback_t value;
/* @@protoc_insertion_point(struct:tensorflow_FloatList) */
} tensorflow_FloatList;

typedef struct _tensorflow_Int64List {
    pb_callback_t value;
/* @@protoc_insertion_point(struct:tensorflow_Int64List) */
} tensorflow_Int64List;

typedef struct _tensorflow_Feature {
    tensorflow_BytesList bytes_list;
    tensorflow_FloatList float_list;
    tensorflow_Int64List int64_list;
/* @@protoc_insertion_point(struct:tensorflow_Feature) */
} tensorflow_Feature;

typedef struct _tensorflow_FeatureLists_FeatureListEntry {
    pb_callback_t key;
    tensorflow_FeatureList value;
/* @@protoc_insertion_point(struct:tensorflow_FeatureLists_FeatureListEntry) */
} tensorflow_FeatureLists_FeatureListEntry;

typedef struct _tensorflow_Features_FeatureEntry {
    pb_callback_t key;
    tensorflow_Feature value;
/* @@protoc_insertion_point(struct:tensorflow_Features_FeatureEntry) */
} tensorflow_Features_FeatureEntry;

/* Default values for struct fields */

/* Initializer values for message structs */
#define tensorflow_BytesList_init_default        {{{NULL}, NULL}}
#define tensorflow_FloatList_init_default        {{{NULL}, NULL}}
#define tensorflow_Int64List_init_default        {{{NULL}, NULL}}
#define tensorflow_Feature_init_default          {tensorflow_BytesList_init_default, tensorflow_FloatList_init_default, tensorflow_Int64List_init_default}
#define tensorflow_Features_init_default         {{{NULL}, NULL}}
#define tensorflow_Features_FeatureEntry_init_default {{{NULL}, NULL}, tensorflow_Feature_init_default}
#define tensorflow_FeatureList_init_default      {{{NULL}, NULL}}
#define tensorflow_FeatureLists_init_default     {{{NULL}, NULL}}
#define tensorflow_FeatureLists_FeatureListEntry_init_default {{{NULL}, NULL}, tensorflow_FeatureList_init_default}
#define tensorflow_BytesList_init_zero           {{{NULL}, NULL}}
#define tensorflow_FloatList_init_zero           {{{NULL}, NULL}}
#define tensorflow_Int64List_init_zero           {{{NULL}, NULL}}
#define tensorflow_Feature_init_zero             {tensorflow_BytesList_init_zero, tensorflow_FloatList_init_zero, tensorflow_Int64List_init_zero}
#define tensorflow_Features_init_zero            {{{NULL}, NULL}}
#define tensorflow_Features_FeatureEntry_init_zero {{{NULL}, NULL}, tensorflow_Feature_init_zero}
#define tensorflow_FeatureList_init_zero         {{{NULL}, NULL}}
#define tensorflow_FeatureLists_init_zero        {{{NULL}, NULL}}
#define tensorflow_FeatureLists_FeatureListEntry_init_zero {{{NULL}, NULL}, tensorflow_FeatureList_init_zero}

/* Field tags (for use in manual encoding/decoding) */
#define tensorflow_BytesList_value_tag           1
#define tensorflow_FeatureList_feature_tag       1
#define tensorflow_FeatureLists_feature_list_tag 1
#define tensorflow_Features_feature_tag          1
#define tensorflow_FloatList_value_tag           1
#define tensorflow_Int64List_value_tag           1
#define tensorflow_Feature_bytes_list_tag        1
#define tensorflow_Feature_float_list_tag        2
#define tensorflow_Feature_int64_list_tag        3
#define tensorflow_FeatureLists_FeatureListEntry_key_tag 1
#define tensorflow_FeatureLists_FeatureListEntry_value_tag 2
#define tensorflow_Features_FeatureEntry_key_tag 1
#define tensorflow_Features_FeatureEntry_value_tag 2

/* Struct field encoding specification for nanopb */
extern const pb_field_t tensorflow_BytesList_fields[2];
extern const pb_field_t tensorflow_FloatList_fields[2];
extern const pb_field_t tensorflow_Int64List_fields[2];
extern const pb_field_t tensorflow_Feature_fields[4];
extern const pb_field_t tensorflow_Features_fields[2];
extern const pb_field_t tensorflow_Features_FeatureEntry_fields[3];
extern const pb_field_t tensorflow_FeatureList_fields[2];
extern const pb_field_t tensorflow_FeatureLists_fields[2];
extern const pb_field_t tensorflow_FeatureLists_FeatureListEntry_fields[3];

/* Maximum encoded size of messages (where known) */
/* tensorflow_BytesList_size depends on runtime parameters */
/* tensorflow_FloatList_size depends on runtime parameters */
/* tensorflow_Int64List_size depends on runtime parameters */
#define tensorflow_Feature_size                  (18 + tensorflow_BytesList_size + tensorflow_FloatList_size + tensorflow_Int64List_size)
/* tensorflow_Features_size depends on runtime parameters */
/* tensorflow_Features_FeatureEntry_size depends on runtime parameters */
/* tensorflow_FeatureList_size depends on runtime parameters */
/* tensorflow_FeatureLists_size depends on runtime parameters */
/* tensorflow_FeatureLists_FeatureListEntry_size depends on runtime parameters */

/* Message IDs (where set with "msgid" option) */
#ifdef PB_MSGID

#define FEATURE_MESSAGES \


#endif

#ifdef __cplusplus
} /* extern "C" */
#endif
/* @@protoc_insertion_point(eof) */

#endif
