/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.3.9 at Wed Mar 28 20:46:46 2018. */

#include "example.pb.h"

/* @@protoc_insertion_point(includes) */
#if PB_PROTO_HEADER_VERSION != 30
#error Regenerate this file with the current version of nanopb generator.
#endif



const pb_field_t tensorflow_Example_fields[2] = {
    PB_FIELD(  1, MESSAGE , SINGULAR, STATIC  , FIRST, tensorflow_Example, features, features, &tensorflow_Features_fields),
    PB_LAST_FIELD
};

const pb_field_t tensorflow_SequenceExample_fields[3] = {
    PB_FIELD(  1, MESSAGE , SINGULAR, STATIC  , FIRST, tensorflow_SequenceExample, context, context, &tensorflow_Features_fields),
    PB_FIELD(  2, MESSAGE , SINGULAR, STATIC  , OTHER, tensorflow_SequenceExample, feature_lists, context, &tensorflow_FeatureLists_fields),
    PB_LAST_FIELD
};


/* Check that field information fits in pb_field_t */
#if !defined(PB_FIELD_32BIT)
/* If you get an error here, it means that you need to define PB_FIELD_32BIT
 * compile-time option. You can do that in pb.h or on compiler command line.
 * 
 * The reason you need to do this is that some of your messages contain tag
 * numbers or field sizes that are larger than what can fit in 8 or 16 bit
 * field descriptors.
 */
PB_STATIC_ASSERT((pb_membersize(tensorflow_Example, features) < 65536 && pb_membersize(tensorflow_SequenceExample, context) < 65536 && pb_membersize(tensorflow_SequenceExample, feature_lists) < 65536), YOU_MUST_DEFINE_PB_FIELD_32BIT_FOR_MESSAGES_tensorflow_Example_tensorflow_SequenceExample)
#endif

#if !defined(PB_FIELD_16BIT) && !defined(PB_FIELD_32BIT)
/* If you get an error here, it means that you need to define PB_FIELD_16BIT
 * compile-time option. You can do that in pb.h or on compiler command line.
 * 
 * The reason you need to do this is that some of your messages contain tag
 * numbers or field sizes that are larger than what can fit in the default
 * 8 bit descriptors.
 */
PB_STATIC_ASSERT((pb_membersize(tensorflow_Example, features) < 256 && pb_membersize(tensorflow_SequenceExample, context) < 256 && pb_membersize(tensorflow_SequenceExample, feature_lists) < 256), YOU_MUST_DEFINE_PB_FIELD_16BIT_FOR_MESSAGES_tensorflow_Example_tensorflow_SequenceExample)
#endif


/* @@protoc_insertion_point(eof) */
